@@ -51,7 +51,14 @@ def make_leaf(url, auth):
 
 class Crawl(object):
 
-    SKIPS = [".*files.*", ".*Individual Files.*", ".*File_Access.*", ".*Forecast Model Run.*", ".*Constant Forecast Offset.*", ".*Constant Forecast Date.*"]
+    SKIPS = [
+        '.*files.*',
+        '.*Individual Files.*',
+        '.*File_Access.*',
+        '.*Forecast Model Run.*',
+        '.*Constant Forecast Offset.*',
+        '.*Constant Forecast Date.*'
+    ]
 
     def __init__(self, catalog_url, select=None, skip=None, before=None, after=None, debug=None, workers=None, auth=None):
         """
@@ -276,12 +283,17 @@ class LeafDataset(object):
                 # Services
                 service_tag = dataset.find("{%s}serviceName" % INV_NS)
                 if service_tag is None:
-                    service_tag = self.metadata.find("{%s}serviceName" % INV_NS)
-                    if service_tag is None:
-                        raise ValueError("No serviceName definition found!")
-                service_name = service_tag.text
+                    if self.metadata is not None:
+                        service_tag = self.metadata.find("{%s}serviceName" % INV_NS)
 
-                for service in tree.findall(".//{%s}service[@name='%s']" % (INV_NS, service_name)):
+                if service_tag is None:
+                    # Use services found in the file. FMRC aggs do this.
+                    services = tree.findall(".//{%s}service[@serviceType='Compound']" % INV_NS)
+                else:
+                    # Use specific named services
+                    services = tree.findall(".//{%s}service[@name='%s']" % (INV_NS, service_tag.text))
+
+                for service in services:
                     if service.get("serviceType") == "Compound":
                         for s in service.findall("{%s}service" % INV_NS):
                             url = construct_url(dataset_url, s.get('base')) + dataset.get("urlPath")
@@ -298,7 +310,7 @@ class LeafDataset(object):
                             url += "?dataset=%s&catalog=%s" % (self.id, quote_plus(self.catalog_url))
                         self.services.append( {'name' : service.get('name'), 'service' : service.get('serviceType'), 'url' : url } )
             except BaseException as e:
-                logger.error('Could not process {}. {}.'.format(dataset_url, e))
+                logger.exception('Could not process {}. {}.'.format(dataset_url, e))
 
     @property
     def size(self):
